import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsBoolean,
  IsDate,
  IsInt,
  IsEnum,
  ValidateNested,
  ValidateIf,
  ArrayMinSize,
  IsNotEmptyObject,
  IsUUID,
} from 'class-validator';
import { Frequency } from '../types/frequency.enum';
import { Transform, Type } from 'class-transformer';
import { mappingKeyToFrequency } from './validators/frequency.mapping';
import { MarginDTO } from './create.margin.dto';
import { ScheduleDTO } from './create.schedule.dto';
import { AddressRequestDTO } from './create.address.request';
import { HasProperPassengerSeats } from './validators/has-passenger-seats.validator';
import { HasProperDriverSeats } from './validators/has-driver-seats.validator';
import { HasProperPositionIndexes } from './validators/address-position.validator';

export class CreateAdRequest {
  @IsOptional()
  @IsUUID(4)
  @AutoMap()
  uuid?: string;

  @IsUUID(4)
  @AutoMap()
  userUuid: string;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => mappingKeyToFrequency(value), {
    toClassOnly: true,
  })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @ValidateIf((ad) => ad.frequency === 'PUNCTUAL')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  departure?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  fromDate?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  toDate?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => ScheduleDTO)
  @ValidateNested({ each: true })
  @IsNotEmptyObject()
  @AutoMap()
  schedule: ScheduleDTO = {};

  @IsOptional()
  @Type(() => MarginDTO)
  @ValidateNested({ each: true })
  @AutoMap()
  marginDurations?: MarginDTO;

  @IsOptional()
  @HasProperDriverSeats()
  @IsInt()
  @AutoMap()
  seatsDriver?: number;

  @IsOptional()
  @HasProperPassengerSeats()
  @IsInt()
  @AutoMap()
  seatsPassenger?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @ArrayMinSize(2)
  @Type(() => AddressRequestDTO)
  @HasProperPositionIndexes()
  @ValidateNested({ each: true })
  @AutoMap()
  addresses: AddressRequestDTO[];
}
